<?php
/**
 * Created by PhpStorm.
 * User: qf
 * Date: 2022-02-25
 * Time: 14:30
 */

if (!defined('IN_DISCUZ')) {
    exit('Access Denied');
}

require_once libfile('function/common', 'plugin/qffloor');

class plugin_qffloor
{
    public $is_open = false;

    public static $identifier = 'qffloor';

    public static $canUseCacheKey = 'can_use_qf_floor';

    public function __construct()
    {
        global $_G;

        if (!$_G['cache']['plugin']['qffloor']) {
            return;
        }

        if ($this->canUseFloor() != 2) {
            return;
        }

        if ($_G['cache']['plugin']['qffloor']) {
            $this->is_open = true;
        }
    }

    /**
     * @return int 1:未拥有  2:已拥有
     */
    protected function canUseFloor()
    {
        $time =  3600;
        if ($use = qianfanCache(self::$canUseCacheKey)) {
            return (int)$use;
        }

        $use = 1;
        $configPath = rtrim(DISCUZ_ROOT, '/') . DIRECTORY_SEPARATOR . 'data' . DIRECTORY_SEPARATOR . 'sysdata' . DIRECTORY_SEPARATOR .
            'cache_qianfan_config.php';
        if (file_exists($configPath)) {
            require($configPath);
            if (isset($qianfan_config['cloud_host']) && isset($qianfan_config['cloud_secretkey'])) {

                $cloudhost = $qianfan_config['cloud_host'] ? $qianfan_config['cloud_host'] : '';
                $cloudsecretkey = $qianfan_config['cloud_secretkey'] ? $qianfan_config['cloud_secretkey'] : '';
                $apiUrl = rtrim($cloudhost, '/') . '/api1_2/thread/can-use-floor';
                $response = $this->qfRequest($cloudsecretkey, $apiUrl, array(), array());

                if ($response && json_decode($response, true) != null) {
                    $resp = json_decode($response, true);
                    if (isset($resp['ret']) && $resp['ret'] == 0) {
                        $use = 2;
                    }
                }
            }
        }
        qianfanCache(self::$canUseCacheKey, $use, $time);

        return (int)$use;
    }

    /**
     * @param $ips
     * @return array|mixed
     */
    public function ipLocate($ips)
    {
        $resp = array();
        if (function_exists('get_ip_locate_floor')) {
            foreach ($ips as $ip) {
                $resp[$ip] = get_ip_locate_floor($ip);
            }
        }
        return $resp;
    }

    protected function qfRequest($secret_key, $url, $get_params, $post_data = array())
    {
        $timeout = 5;
        $nonce = rand(10000, 99999);
        $timestamp = time();
        $array = array($nonce, $timestamp, $secret_key);
        sort($array, SORT_STRING);
        $token = md5(implode($array));
        $params['nonce'] = $nonce;
        $params['timestamp'] = $timestamp;
        $params['token'] = $token;
        $params = array_merge($params, $get_params);
        $url .= '?';
        foreach ($params as $k => $v) {
            $url .= $k . '=' . $v . '&';
        }
        $url = rtrim($url, '&');
        $curlHandle = curl_init();
        curl_setopt($curlHandle, CURLOPT_URL, $url);
        curl_setopt($curlHandle, CURLOPT_HEADER, 0);
        curl_setopt($curlHandle, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curlHandle, CURLOPT_USERAGENT, 'bbs, qianfan');

        curl_setopt($curlHandle, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curlHandle, CURLOPT_SSL_VERIFYHOST, FALSE);
        curl_setopt($curlHandle, CURLOPT_POST, count($post_data));
        curl_setopt($curlHandle, CURLOPT_POSTFIELDS, $post_data);
        curl_setopt($curlHandle, CURLOPT_CONNECTTIMEOUT, $timeout);
        curl_setopt($curlHandle, CURLOPT_TIMEOUT, $timeout);

        $data = curl_exec($curlHandle);
        $status = curl_getinfo($curlHandle, CURLINFO_HTTP_CODE);
        if ($status != 200) {
            return false;
        }
        curl_close($curlHandle);
        return $data;
    }
}

class plugin_qffloor_forum extends plugin_qffloor
{
    // 列表中的帖子评论数量
    function forumdisplay_thread_output()
    {
        global $_G;
        if ($this->is_open) {
            $tids = array();
            $count = array();
            foreach ($_G['forum_threadlist'] as $key => $value) {
                $tids[] = $value['tid'];
            }
            if ($tids) {
                $floorStats = C::t('#' . self::$identifier . '#qianfan_post_floor_stats')->fetch_approve_num_by_tid($tids);
                foreach ($floorStats as $value) {
                    $count[$value['tid']] = $value['approve_num'];
                }
                foreach ($_G['forum_threadlist'] as &$thread) {
                    if ($count[$thread['tid']]) {
                        $thread['allreplies'] += $count[$thread['tid']];
                        $thread['replies'] += $count[$thread['tid']];
                    }
                }
            }
        }
        return null;
    }

    // 引入css
    function viewthread_top_output()
    {
        global $_G, $allowpostreply;
        if ($this->is_open) {
            $setconfig = $_G['cache']['plugin']['qffloor'];
            $setconfig['replace_pc'] = 2;
            $setconfig['style_pc'] = 1;
            include template(self::$identifier . ':style');
        }
        return $return;
    }

    // 引入js
    function viewthread_bottom_output()
    {
        global $_G, $allowpostreply;
        if ($this->is_open) {
            $setconfig = $_G['cache']['plugin']['qffloor'];
            $setconfig['replace_pc'] = 2;
            $setconfig['style_pc'] = 1;
            include template(self::$identifier . ':js');
        }
        return $return;
    }


    // 帖子楼中楼回复列表
    function viewthread_postbottom_output()
    {
        global $_G, $postlist, $postno, $postnostick;
        $t = $pids = array();
        if ($_G['inajax']) {
            return $t;
        }
        if ($this->is_open) {
            $setconfig = $_G['cache']['plugin']['qffloor'];
            $setconfig['replace_pc'] = 2;
            $setconfig['style_pc'] = 1;

            foreach ($postlist as $key => $post) {
                if (!$post['first']) {
                    $pids[] = $post['pid'];
                }
            }
            $pageCount = (int)$_G['cache']['plugin']['qffloor']['page_count'];
            if ($pids) {
                if (isset($_G['thread']['tid']) && isset($_G['thread']['replies'])) {
                    $tids = array($_G['thread']['tid']);
                    $floorStats = C::t('#' . self::$identifier . '#qianfan_post_floor_stats')->fetch_approve_num_by_tid($tids);
                    foreach ($floorStats as $value) {
                        if ($_G['thread']['tid'] == $value['tid']) {
                            $_G['thread']['allreplies'] += $value['approve_num'];
                            $_G['thread']['replies'] += $value['approve_num'];
                        }
                    }
                }

                $order = 'order by created_at asc ';
                list($messageinfo, $ipArr) = C::t('#' . self::$identifier . '#qianfan_post_floor')->fetch_list_by_pids($pids, $order, $pageCount);

                $locateArr = array();
                if (!empty($setconfig['enable_ip_locate']) && $ipArr) {
                    $ipArr = array_values(array_unique($ipArr));
                    $locateArr = $this->ipLocate($ipArr);
                }

                if ($messageinfo && is_array($messageinfo)) {
                    foreach ($messageinfo as &$msg) {
                        if (!empty($msg['list'])) {
                            foreach ($msg['list'] as &$v) {
                                if (isset($locateArr[$v['useip']])) {
                                    $locate = strtoupper($_G['charset']) == 'UTF-8' ? $locateArr[$v['useip']] : diconv($locateArr[$v['useip']], 'UTF-8', $_G['charset']);
                                    $v['locate'] = $locate;
                                } else {
                                    $v['locate'] = '';
                                }
                            }
                        }
                    }
                }

                foreach ($postlist as $key => $post) {
                    if ($post['first']) {
                        $t[] = '';
                    } else {
                        $count = $messageinfo[$post['pid']]['count'] ? $messageinfo[$post['pid']]['count'] : 0;
                        $list = $messageinfo[$post['pid']]['list'];
                        $_GET['ajaxtarget'] = 'replyfloor_content_' . $post['pid'];
                        $multipage = multi($count, $pageCount, 1, 'plugin.php?id=qffloor:index&tid=' . $post['tid'] . '&pid=' . $post['pid']);
                        $_GET['ajaxtarget'] = '';
                        include template(self::$identifier . ':viewthread');
                        $t[] = $return;
                    }
                }
            }
        }
        return $t;
    }


}